''' <summary>
''' Base form to contain common code for all forms
''' Example 5
''' Demonstrates a generic Save method
''' Example 6
''' Demonstrates a generic Compare method
''' </summary>
Public Class BaseWin

#Region " Protected Methods"

#Region "Save"
    ''' <summary>
    ''' Performs the save for the form
    ''' </summary>
    ''' <typeparam name="T">Data type of the object to save</typeparam>
    ''' <param name="objectToSave">Object to save</param>
    Protected Sub Save(Of T)(ByVal objectToSave As T)
        ' Do validation, etc

        ' Call the appropriate save method
        Dim _methodInfo As Reflection.MethodInfo
        _methodInfo = GetType(T).GetMethod("Save")

        _methodInfo.Invoke(objectToSave, Nothing)

        ' Do any post processing
    End Sub
#End Region

#End Region

End Class

#Region " IComparer"
''' <summary>
''' Supports sorting
''' </summary>
''' <typeparam name="T">Data type of the objects to sort</typeparam>
<System.ComponentModel.Browsable(False)> _
Public Class PropertyComparer(Of T)
    Implements IComparer(Of T)

    Private ReadOnly _propertyInfo As Reflection.PropertyInfo
    Private ReadOnly _sortDirection As SortOrder

    Public Sub New(ByVal propertyToSort As String, ByVal sortDirection As SortOrder)
        _sortDirection = sortDirection
        _propertyInfo = GetType(T).GetProperty(propertyToSort, _
            Reflection.BindingFlags.Instance Or Reflection.BindingFlags.Public _
            Or Reflection.BindingFlags.FlattenHierarchy Or Reflection.BindingFlags.IgnoreCase)
    End Sub

    Public Function Compare(ByVal x As T, ByVal y As T) As Integer Implements IComparer(Of T).Compare
        Dim xValue As Object = _propertyInfo.GetValue(x, Nothing)
        Dim yValue As Object = _propertyInfo.GetValue(y, Nothing)
        If _sortDirection = SortOrder.Ascending Then
            Return System.Collections.Comparer.Default.Compare(xValue, yValue)
        Else
            Return System.Collections.Comparer.Default.Compare(yValue, xValue)
        End If
    End Function
End Class

#End Region